import * as fs from "fs";
import * as path from "path";
import { program } from "commander";
import { CassetteSpeed, decodeBasicProgram, decodeSystemProgram, decodeTrs80CassetteFile, decodeTrs80File, decodeTrsdos, getTrs80FileExtension, isFloppy, trsdosProtectionLevelToString } from "trs80-base";
import { concatByteArrays, withCommas } from "teamten-ts-utils";
import { binaryAsCasFile, BitType, casAsAudio, concatAudio, Decoder, DEFAULT_SAMPLE_RATE, readWavFile, Tape, writeWavFile } from "trs80-cassette";
import { version } from "./version.js";
import { disasmForTrs80Program } from "trs80-disasm";
import { instructionsToText } from "z80-disasm";
const HELP_TEXT = `
See this page for full documentation:

https://github.com/lkesteloot/trs80/blob/master/packages/trs80-tool/README.md
`;
/**
 * Return the singular or plural version of a string depending on the count.
 */
function pluralize(count, singular, plural) {
    return count === 1 ? singular : plural !== null && plural !== void 0 ? plural : (singular + "s");
}
/**
 * Return the count and the singular or plural version of a string depending on the count.
 */
function pluralizeWithCount(count, singular, plural) {
    return `${count} ${pluralize(count, singular, plural)}`;
}
/**
 * Super class for files that are nested in another file, such as a floppy or cassette.
 */
class ArchiveFile {
    constructor(filename, date) {
        this.filename = filename;
        this.date = date;
    }
}
/**
 * Nested file that came from a cassette file.
 */
class WavFile extends ArchiveFile {
    constructor(program) {
        super(WavFile.getFilename(program), undefined);
        this.program = program;
    }
    static getFilename(program) {
        let extension = ".BIN";
        if (program.isEdtasmProgram()) {
            extension = ".ASM";
        }
        else if (program.isBasicProgram()) {
            extension = ".BAS";
        }
        else if (decodeSystemProgram(program.binary) !== undefined) {
            extension = ".3BN";
        }
        return program.getPseudoFilename() + extension;
    }
    getDirString() {
        return this.filename.padEnd(12) + " " +
            withCommas(this.program.binary.length).padStart(8) + "  " +
            this.program.baud + " baud";
    }
    getBinary() {
        return this.program.binary;
    }
}
/**
 * Nested file that came from a CAS file.
 */
class CasFile extends ArchiveFile {
    constructor(filename, cassetteFile) {
        super(filename, undefined);
        this.cassetteFile = cassetteFile;
    }
    getBinary() {
        return this.cassetteFile.file.binary;
    }
    getDirString() {
        return this.filename.padEnd(14) + " " +
            withCommas(this.cassetteFile.file.binary.length).padStart(8) + "  " +
            (this.cassetteFile.speed === CassetteSpeed.HIGH_SPEED ? "1500" : "500") + " baud";
    }
}
/**
 * Nested file that came from a TRSDOS floppy.
 */
class TrsdosFile extends ArchiveFile {
    constructor(trsdos, file) {
        super(file.getFilename("."), file.getDate());
        this.trsdos = trsdos;
        this.file = file;
    }
    getDirString() {
        return this.file.getFilename(".").padEnd(12) + " " +
            withCommas(this.file.getSize()).padStart(8) + " " +
            this.file.getDateString() + " " +
            trsdosProtectionLevelToString(this.file.getProtectionLevel());
    }
    getBinary() {
        return this.trsdos.readFile(this.file);
    }
}
/**
 * List of nested files in an archive (floppy or cassette).
 * TODO: I think we can get rid of this and use an array of InputFile objects instead.
 */
class Archive {
    constructor(filename) {
        this.files = [];
        // Read the file.
        let buffer;
        try {
            buffer = fs.readFileSync(filename);
        }
        catch (e) {
            this.error = "Can't open \"" + filename + "\": " + e.message;
            return;
        }
        if (filename.toLowerCase().endsWith(".wav")) {
            // Decode the cassette.
            const wavFile = readWavFile(buffer.buffer);
            this.tape = new Tape(filename, wavFile);
            const decoder = new Decoder(this.tape);
            decoder.decode();
            for (const program of this.tape.programs) {
                this.files.push(new WavFile(program));
            }
        }
        else {
            // Decode the floppy or cassette.
            const file = decodeTrs80File(buffer, filename);
            switch (file.className) {
                case "Jv1FloppyDisk":
                case "Jv3FloppyDisk":
                case "DmkFloppyDisk":
                    const trsdos = decodeTrsdos(file);
                    if (trsdos !== undefined) {
                        for (const dirEntry of trsdos.dirEntries) {
                            this.files.push(new TrsdosFile(trsdos, dirEntry));
                        }
                    }
                    else {
                        this.error = "Can only handle TRSDOS floppies.";
                    }
                    break;
                case "Cassette":
                    let counter = 1;
                    const name = path.parse(filename).name;
                    for (const cassetteFile of file.files) {
                        const cassetteFilename = name + "-T" + counter + getTrs80FileExtension(cassetteFile.file);
                        this.files.push(new CasFile(cassetteFilename, cassetteFile));
                        counter += 1;
                    }
                    break;
                default:
                    this.error = "This file type (" + file.className + ") does not have nested files.";
                    break;
            }
        }
    }
    /**
     * Get the file for the given filename, which should have a dot separator for the extension.
     */
    getFileByFilename(filename) {
        filename = filename.toUpperCase();
        for (const file of this.files) {
            if (file.filename === filename) {
                return file;
            }
        }
        return undefined;
    }
}
/**
 * Information about each input file.
 */
class InputFile {
    constructor(filename, trs80File, baud, date) {
        this.filename = filename;
        this.trs80File = trs80File;
        this.baud = baud;
        this.date = date;
    }
    /**
     * Return a new InputFile but with the file replaced by the parameter.
     */
    withFile(trs80File) {
        return new InputFile(this.filename, trs80File, this.baud, this.date);
    }
}
/**
 * Get a one-line string description of the input file.
 */
function getInfoForFile(filename) {
    const { base, name, ext } = path.parse(filename);
    let buffer;
    try {
        buffer = fs.readFileSync(filename);
    }
    catch (e) {
        return "Can't open file: " + e.message;
    }
    if (ext.toLowerCase() == ".wav") {
        // Parse a cassette WAV file.
        const wavFile = readWavFile(buffer.buffer);
        const tape = new Tape(base, wavFile);
        const decoder = new Decoder(tape);
        decoder.decode();
        return "Audio file with " + pluralizeWithCount(tape.programs.length, "file");
    }
    else {
        const trs80File = decodeTrs80File(buffer, filename);
        if (trs80File.error !== undefined) {
            return trs80File.error;
        }
        return trs80File.getDescription();
    }
}
/**
 * Handle the "dir" command.
 */
function dir(infile) {
    const archive = new Archive(infile);
    if (archive.error !== undefined) {
        console.log(archive.error);
    }
    else {
        for (const file of archive.files) {
            console.log(file.getDirString());
        }
    }
}
/**
 * Handle the "info" command.
 */
function info(infiles) {
    for (const infile of infiles) {
        console.log(infile + ": " + getInfoForFile(infile));
    }
}
/**
 * Handle the "extract" command.
 */
function extract(infile, outfile) {
    const archive = new Archive(infile);
    if (archive.error !== undefined) {
        console.log(archive.error);
        process.exit(1);
    }
    else {
        // See if outfile is an existing directory.
        if (fs.existsSync(outfile) && fs.statSync(outfile).isDirectory()) {
            // Extract all files to this directory.
            for (const file of archive.files) {
                const binary = file.getBinary();
                const outPathname = path.join(outfile, file.filename);
                fs.writeFileSync(outPathname, binary);
                if (file.date !== undefined) {
                    fs.utimesSync(outPathname, file.date, file.date);
                }
                console.log("Extracted " + file.filename + " to " + outPathname);
            }
        }
        else {
            // Break apart outfile.
            const { dir, base, name, ext } = path.parse(outfile);
            // See if it's a JSON file.
            if (ext.toLowerCase() === ".json") {
                // Output metadata to JSON file.
                const fullData = {
                    programs: [],
                    version: 1,
                };
                if (archive.tape !== undefined) {
                    fullData.sampleRate = archive.tape.sampleRate;
                }
                for (let i = 0; i < archive.files.length; i++) {
                    const file = archive.files[i];
                    const programData = {
                        name: file.filename,
                    };
                    fullData.programs.push(programData);
                    if (file instanceof WavFile) {
                        const program = file.program;
                        programData.trackNumber = program.trackNumber;
                        programData.copyNumber = program.copyNumber;
                        programData.startFrame = program.startFrame;
                        programData.endFrame = program.endFrame;
                        programData.speed = program.baud;
                        programData.length = program.binary.length;
                        // Decode various formats.
                        programData.type = "unknown";
                        // Analyze system program.
                        const fileBinary = file.getBinary();
                        const systemProgram = decodeSystemProgram(fileBinary);
                        if (systemProgram !== undefined) {
                            programData.type = "systemProgram";
                            programData.filename = systemProgram.filename;
                            programData.chunkCount = systemProgram.chunks.length;
                            // Check for checksum errors.
                            let checksumErrors = 0;
                            for (const chunk of systemProgram.chunks) {
                                if (!chunk.isChecksumValid()) {
                                    checksumErrors += 1;
                                }
                            }
                            programData.checksumErrorCount = checksumErrors;
                        }
                        // Label Basic program.
                        const basicProgram = decodeBasicProgram(fileBinary);
                        if (basicProgram !== undefined) {
                            programData.type = "basic";
                        }
                        // Warn about bit errors.
                        programData.errorCount = program.countBitErrors();
                        programData.errors = [];
                        for (const bitData of program.bitData) {
                            if (bitData.bitType === BitType.BAD) {
                                programData.errors.push(Math.round((bitData.startFrame + bitData.endFrame) / 2));
                            }
                        }
                        // See if it's a duplicate.
                        let isDuplicate = false;
                        for (let j = 0; j < i; j++) {
                            const otherProgram = archive.files[j].program;
                            if (program.sameBinaryAs(otherProgram)) {
                                isDuplicate = true;
                                break;
                            }
                        }
                        programData.isDuplicate = isDuplicate;
                    }
                    if (file instanceof TrsdosFile) {
                        programData.date = file.file.getDateString();
                        programData.timestamp = file.file.getDate().getTime();
                        programData.protectionLevel = trsdosProtectionLevelToString(file.file.getProtectionLevel());
                        programData.size = file.file.getSize();
                        programData.isSystemFile = file.file.isSystemFile();
                        programData.isExtendedEntry = file.file.isExtendedEntry();
                        programData.isHidden = file.file.isHidden();
                        programData.isActive = file.file.isActive();
                    }
                }
                fs.writeFileSync(outfile, JSON.stringify(fullData, undefined, 2));
                console.log("Generated " + outfile);
            }
            else {
                // Output file contents to file.
                const file = archive.getFileByFilename(base);
                if (file === undefined) {
                    // TODO: Look by name and different extension.
                    console.log("Can't find file " + base);
                }
                else {
                    const binary = file.getBinary();
                    fs.writeFileSync(outfile, binary);
                    if (file.date !== undefined) {
                        fs.utimesSync(outfile, file.date, file.date);
                    }
                    console.log("Extracted " + file.filename + " to " + outfile);
                }
            }
        }
    }
}
/**
 * Disassemble a program.
 *
 * @param trs80File program to disassemble.
 * @param entryPoints additional entry points in binary.
 */
function disassemble(trs80File, entryPoints) {
    const disasm = disasmForTrs80Program(trs80File);
    for (const entryPoint of entryPoints) {
        disasm.addEntryPoint(entryPoint);
    }
    const instructions = disasm.disassemble();
    const text = instructionsToText(instructions).join("\n") + "\n";
    const outBinary = new TextEncoder().encode(text);
    const description = "Disassembled " + (trs80File.className === "CmdProgram" ? "CMD program" : "system program");
    return [outBinary, description];
}
/**
 * Handle the "convert" command.
 *
 * @param inFilenames list of input filenames.
 * @param outFilename single output filename or directory.
 * @param baud optional new baud rate.
 * @param start optional start address of system file.
 * @param entryPoints additional entry points in binary.
 */
function convert(inFilenames, outFilename, baud, start, entryPoints) {
    var _a, _b, _c, _d, _e, _f, _g;
    const inFiles = [];
    // Read all input files into an internal data structure, expanding archives like cassettes and floppies.
    for (const inFilename of inFilenames) {
        const { base, name, ext } = path.parse(inFilename);
        let buffer;
        try {
            buffer = fs.readFileSync(inFilename);
        }
        catch (e) {
            console.log("Can't open \"" + inFilename + "\": " + e.message);
            process.exit(1);
        }
        if (ext.toLowerCase() == ".wav") {
            // Parse a cassette WAV file.
            const wavFile = readWavFile(buffer.buffer);
            const tape = new Tape(base, wavFile);
            const decoder = new Decoder(tape);
            decoder.decode();
            for (const program of tape.programs) {
                const trs80File = decodeTrs80CassetteFile(program.binary);
                const filename = name + "-" + program.getPseudoFilename() + getTrs80FileExtension(trs80File);
                inFiles.push(new InputFile(filename, trs80File, program.baud));
            }
        }
        else {
            const trs80File = decodeTrs80File(buffer, inFilename);
            if (trs80File.error !== undefined) {
                console.log("Can't open \"" + inFilename + "\": " + trs80File.error);
                process.exit(1);
            }
            if (isFloppy(trs80File)) {
                const trsdos = decodeTrsdos(trs80File);
                if (trsdos === undefined) {
                    // Should probably fail here. Having the floppy disk itself is probably not
                    // what the user wants.
                    inFiles.push(new InputFile(base, trs80File));
                }
                else {
                    // Expand floppy.
                    for (const dirEntry of trsdos.dirEntries) {
                        const trsdosFilename = dirEntry.getFilename(".");
                        const trsdosBinary = trsdos.readFile(dirEntry);
                        const trsdosTrs80File = decodeTrs80File(trsdosBinary, trsdosFilename);
                        inFiles.push(new InputFile(trsdosFilename, trsdosTrs80File, undefined, dirEntry.getDate()));
                    }
                }
            }
            else if (trs80File.className === "Cassette") {
                // Expand .CAS file.
                let counter = 1;
                for (const cassetteFile of trs80File.files) {
                    const filename = name + "-T" + counter + getTrs80FileExtension(cassetteFile.file);
                    const baud = cassetteFile.speed === CassetteSpeed.LOW_SPEED ? 500 : 1500;
                    inFiles.push(new InputFile(filename, cassetteFile.file, baud));
                    counter++;
                }
            }
            else {
                inFiles.push(new InputFile(base, trs80File));
            }
        }
    }
    // Update start address if requested.
    if (start !== undefined) {
        for (let i = 0; i < inFiles.length; i++) {
            const trs80File = inFiles[i].trs80File;
            let newTrs80File;
            if (trs80File.className === "SystemProgram") {
                if (start === "auto") {
                    if (trs80File.entryPointAddress === 0) {
                        const guessAddress = trs80File.guessEntryAddress();
                        if (guessAddress !== undefined) {
                            newTrs80File = trs80File.withEntryPointAddress(guessAddress);
                        }
                    }
                }
                else {
                    newTrs80File = trs80File.withEntryPointAddress(start);
                }
            }
            if (newTrs80File !== undefined) {
                inFiles[i] = inFiles[i].withFile(newTrs80File);
            }
        }
    }
    // If output is existing directory, put all input files there.
    if (fs.existsSync(outFilename) && fs.statSync(outFilename).isDirectory()) {
        for (const infile of inFiles) {
            const outpath = path.join(outFilename, infile.filename);
            fs.writeFileSync(outpath, infile.trs80File.binary);
            if (infile.date !== undefined) {
                fs.utimesSync(outpath, infile.date, infile.date);
            }
            console.log("Wrote " + outpath + ": " + infile.trs80File.getDescription());
        }
    }
    else {
        // Output is a file. Its extension will help us determine how to convert the input files.
        const outExt = path.parse(outFilename).ext.toLowerCase();
        if (outExt === "") {
            console.log("No file extension on output file \"" + outFilename + "\", don't know how to convert");
            process.exit(1);
        }
        // See if input is a single file.
        if (inFiles.length === 1) {
            // Convert individual file.
            const infile = inFiles[0];
            const inName = path.parse(infile.filename).name;
            let outBinary;
            let description;
            switch (infile.trs80File.className) {
                case "RawBinaryFile":
                    console.log("Cannot convert unknown file type of " + infile.filename);
                    process.exit(1);
                    break;
                case "BasicProgram":
                    switch (outExt) {
                        case ".bas":
                            // Write as-is.
                            outBinary = infile.trs80File.binary;
                            description = "Basic file (tokenized)";
                            break;
                        case ".asc":
                            // Convert to ASCII.
                            outBinary = Buffer.from(infile.trs80File.asAscii());
                            description = "Basic file (plain text)";
                            break;
                        case ".cas": {
                            // Encode in CAS file.
                            const outBaud = (_a = (baud !== null && baud !== void 0 ? baud : infile.baud)) !== null && _a !== void 0 ? _a : 500;
                            outBinary = binaryAsCasFile(infile.trs80File.asCassetteBinary(), outBaud);
                            description = "Basic file in " + (outBaud >= 1500 ? "high" : "low") + " speed CAS file";
                            break;
                        }
                        case ".wav": {
                            // Encode in WAV file.
                            const outBaud = (_b = (baud !== null && baud !== void 0 ? baud : infile.baud)) !== null && _b !== void 0 ? _b : 500;
                            const cas = binaryAsCasFile(infile.trs80File.asCassetteBinary(), outBaud);
                            const audio = casAsAudio(cas, outBaud, DEFAULT_SAMPLE_RATE);
                            outBinary = writeWavFile(audio, DEFAULT_SAMPLE_RATE);
                            description = "Basic file in " + baud + " baud WAV file";
                            break;
                        }
                        default:
                            console.log("Can't convert a Basic program to " + outExt.toUpperCase());
                            process.exit(1);
                            break;
                    }
                    break;
                case "Jv1FloppyDisk":
                case "Jv3FloppyDisk":
                case "DmkFloppyDisk":
                case "Cassette":
                    console.log("Files of type \"" + infile.trs80File.getDescription + "\" are not yet supported");
                    process.exit(1);
                    break;
                case "SystemProgram":
                    switch (outExt) {
                        case ".3bn":
                            // Write as-is.
                            outBinary = infile.trs80File.binary;
                            description = infile.trs80File.getDescription();
                            break;
                        case ".cmd": {
                            // Convert to CMD program.
                            const cmdProgram = infile.trs80File.toCmdProgram(inName.toUpperCase());
                            outBinary = cmdProgram.binary;
                            description = cmdProgram.getDescription();
                            break;
                        }
                        case ".cas": {
                            // Encode in CAS file.
                            const outBaud = (_c = (baud !== null && baud !== void 0 ? baud : infile.baud)) !== null && _c !== void 0 ? _c : 500;
                            outBinary = binaryAsCasFile(infile.trs80File.binary, outBaud);
                            description = infile.trs80File.getDescription() + " in " +
                                (outBaud >= 1500 ? "high" : "low") + " speed CAS file";
                            break;
                        }
                        case ".wav": {
                            // Encode in WAV file.
                            const outBaud = (_d = (baud !== null && baud !== void 0 ? baud : infile.baud)) !== null && _d !== void 0 ? _d : 500;
                            const cas = binaryAsCasFile(infile.trs80File.binary, outBaud);
                            const audio = casAsAudio(cas, outBaud, DEFAULT_SAMPLE_RATE);
                            outBinary = writeWavFile(audio, DEFAULT_SAMPLE_RATE);
                            description = infile.trs80File.getDescription() + " in " + outBaud + " baud WAV file";
                            break;
                        }
                        case ".lst":
                            [outBinary, description] = disassemble(infile.trs80File, entryPoints);
                            break;
                        default:
                            console.log("Can't convert a system program program to " + outExt.toUpperCase());
                            process.exit(1);
                            break;
                    }
                    break;
                case "CmdProgram":
                    switch (outExt) {
                        case ".cmd":
                            // Write as-is.
                            outBinary = infile.trs80File.binary;
                            description = infile.trs80File.getDescription();
                            break;
                        case ".3bn": {
                            // Convert to system program.
                            const systemProgram = infile.trs80File.toSystemProgram(inName.toUpperCase());
                            outBinary = systemProgram.binary;
                            description = systemProgram.getDescription();
                            break;
                        }
                        case ".cas": {
                            // Encode in CAS file.
                            const outBaud = (_e = (baud !== null && baud !== void 0 ? baud : infile.baud)) !== null && _e !== void 0 ? _e : 500;
                            const systemProgram = infile.trs80File.toSystemProgram(inName.toUpperCase());
                            const sysBinary = systemProgram.binary;
                            outBinary = binaryAsCasFile(sysBinary, outBaud);
                            description = systemProgram.getDescription() + " in " +
                                (outBaud >= 1500 ? "high" : "low") + " speed CAS file";
                            break;
                        }
                        case ".wav": {
                            // Encode in WAV file.
                            const outBaud = (_f = (baud !== null && baud !== void 0 ? baud : infile.baud)) !== null && _f !== void 0 ? _f : 500;
                            const systemProgram = infile.trs80File.toSystemProgram(inName.toUpperCase());
                            const sysBinary = systemProgram.binary;
                            const cas = binaryAsCasFile(sysBinary, outBaud);
                            const audio = casAsAudio(cas, outBaud, DEFAULT_SAMPLE_RATE);
                            outBinary = writeWavFile(audio, DEFAULT_SAMPLE_RATE);
                            description = systemProgram.getDescription() + " in " + outBaud + " baud WAV file";
                            break;
                        }
                        case ".lst":
                            [outBinary, description] = disassemble(infile.trs80File, entryPoints);
                            break;
                        default:
                            console.log("Can't convert a CMD program to " + outExt.toUpperCase());
                            process.exit(1);
                            break;
                    }
                    break;
            }
            fs.writeFileSync(outFilename, outBinary);
            console.log("Wrote " + outFilename + ": " + description);
        }
        else {
            // Make archive.
            if (outExt === ".cas" || outExt === ".wav") {
                // Make the individual cas files and their baud rate.
                const outCasFiles = [];
                for (const inFile of inFiles) {
                    // Convert to cassette format if necessary.
                    let outBinary;
                    let description;
                    switch (inFile.trs80File.className) {
                        case "RawBinaryFile":
                        case "SystemProgram":
                            // Keep as-is.
                            outBinary = inFile.trs80File.binary;
                            description = inFile.trs80File.getDescription();
                            break;
                        case "BasicProgram":
                            outBinary = inFile.trs80File.asCassetteBinary();
                            description = inFile.trs80File.getDescription();
                            break;
                        case "Jv1FloppyDisk":
                        case "Jv3FloppyDisk":
                        case "DmkFloppyDisk":
                        case "Cassette":
                            // Shouldn't happen, we split up archives.
                            console.log(`Can't put ${inFile.trs80File.getDescription()} into a ${outExt.toUpperCase()} file`);
                            process.exit(1);
                            break;
                        case "CmdProgram": {
                            // Convert to system program first.
                            const inName = path.parse(inFile.filename).name;
                            const systemProgram = inFile.trs80File.toSystemProgram(inName.toUpperCase());
                            outBinary = systemProgram.binary;
                            description = systemProgram.getDescription();
                            break;
                        }
                    }
                    const outBaud = (_g = (baud !== null && baud !== void 0 ? baud : inFile.baud)) !== null && _g !== void 0 ? _g : 500;
                    outCasFiles.push({ cas: binaryAsCasFile(outBinary, outBaud), baud: outBaud });
                    console.log("In output file: " + description);
                }
                if (outExt === ".cas") {
                    fs.writeFileSync(outFilename, concatByteArrays(outCasFiles.map(outCas => outCas.cas)));
                    console.log("Wrote " + outFilename + ": CAS file with " +
                        pluralizeWithCount(outCasFiles.length, "file"));
                }
                else {
                    // outExt == ".wav"
                    const audioParts = [];
                    for (const outCas of outCasFiles) {
                        // One second of silence before each program.
                        audioParts.push(new Int16Array(DEFAULT_SAMPLE_RATE));
                        // Convert program to audio.
                        audioParts.push(casAsAudio(outCas.cas, outCas.baud, DEFAULT_SAMPLE_RATE));
                        // One second of silence after each program.
                        audioParts.push(new Int16Array(DEFAULT_SAMPLE_RATE));
                    }
                    const wavBinary = writeWavFile(concatAudio(audioParts), DEFAULT_SAMPLE_RATE);
                    fs.writeFileSync(outFilename, wavBinary);
                    console.log("Wrote " + outFilename + ": WAV file with " +
                        pluralizeWithCount(outCasFiles.length, "file"));
                }
            }
            else {
                // TODO handle floppy.
                console.log("Can't put multiple files into " + outExt.toUpperCase());
            }
        }
    }
}
/**
 * Handle the "hexdump" command.
 */
function hexdump(infile) {
}
function main() {
    program
        .storeOptionsAsProperties(false)
        .name("trs80-tool")
        .addHelpText("after", HELP_TEXT)
        .version(version);
    program
        .command("dir <infile>")
        .description("list files in the infile", {
        infile: "WAV, CAS, JV1, JV3, or DMK file (TRSDOS floppies only)",
    })
        .action(infile => {
        dir(infile);
    });
    program
        .command("info <infiles...>")
        .description("show information about each file", {
        infile: "any TRS-80 file",
    })
        .action(infiles => {
        info(infiles);
    });
    /*
    program
        .command("extract <infile> <outfile>")
        .description("extract files in the infile", {
            infile: "WAV, CAS, JV1, JV3, or DMK file (TRSDOS floppies only)",
            outfile: "path to file or directory, or to JSON file for metadata",
        })
        .action((infile, outfile) => {
            extract(infile, outfile);
        });*/
    program
        .command("convert <files...>")
        .description("convert one or more infiles to one outfile", {
        infile: "WAV, CAS, CMD, 3BN, or BAS file",
        outfile: "WAV, CAS, CMD, 3BN, BAS, or LST file",
    })
        .option("--baud <baud>", "output baud rate (250, 500, 1000, or 1500)")
        .option("--start <address>", "new start address of system program, or \"auto\" to guess")
        .option("--entry <addresses>", "add entry points of binary (comma-separated), for LST output")
        .action((files, options) => {
        const baud = options.baud !== undefined ? parseInt(options.baud) : undefined;
        const start = options.start !== undefined ? options.start === "auto" ? "auto" : parseInt(options.start) : undefined;
        const entryPoints = options.entry !== undefined ? options.entry.split(",").map(x => parseInt(x)) : [];
        if (files.length < 2) {
            console.log("Must specify at least one infile and exactly one outfile");
            process.exit(1);
        }
        const infiles = files.slice(0, files.length - 1);
        const outfile = files[files.length - 1];
        convert(infiles, outfile, baud, start, entryPoints);
    });
    /*
    program
        .command("hexdump <infile>")
        .description("display an annotated hexdump of infile", {
            infile: "WAV, CAS, CMD, 3BN, BAS, JV1, JV3, or DMK",
        })
        .action(infile => {
            hexdump(infile);
        });*/
    program
        .parse(process.argv);
}
main();
//# sourceMappingURL=index.js.map